package readerwriter;

/**
 * <p>Title:        ReaderWriter
 * <p>Description:  Progettazione di un mini framework per la gestione generale
 * di risorse condivise in maniera concorrente in lettura/scrittura
 *
 * <p>Copyright:    Copyright (c) 2001
 * <p>Company:      Universita' degli studi di Milano (Facolta' di Informatica)
 *
 * @author Gabriele "SunGray" Lana
 * @version 1.0
 */

/**
 * La classe concrete definiranno le priorita' di accesso alla risorsa condivisa, da parte di Lettori e
 * Scrittori, in base alla situazione esistente : Lettori/Scrittori che gia' occupano la risorsa,
 * Lettori/Scrittori che sono in attesa dell'utilizzo della risorsa.
 * La classe astratta Guardian, implementa i metodi base di gestione dei lettori/scrittori per una risorsa
 * condivisa accessibile in lettura (inizioLettura/fineLettura) ed in scrittura
 * (inizioScrittura/fineScrittura)
 */

public abstract class Guardian
{ private int numReaders;
  private int numWriters;

  /**
   * Inizializza allo stato di default le variabili private della classe
   */
  public Guardian()
  { numReaders = 0;
    numWriters = 0;
  }


  /**
   * Determina l'inizio dell'operazione di lettura da parte di un thread
   *
   * @param priority L'ogetto che rappresenta la priorita' da attribuire all'operazione
   * @exception SemaphoredThreadRequireException Eccezione lanciata se il thread che sta per eseguire il
   * metodo non e' un Thread di tipo <code>SemaphoredThread</code>
   */
  public void iniziaLettura(Object priority) throws SemaphoredThreadRequireException
  { }


  /**
   * Determina la fine dell'operazione di lettura da parte di un thread
   *
   * @exception SemaphoredThreadRequireException Eccezione lanciata se il thread che sta per eseguire il
   * metodo non e' un Thread di tipo <code>SemaphoredThread</code>
   */
  public void fineLettura() throws SemaphoredThreadRequireException
  { }


  /**
   * Determina l'inizio dell'operazione di scrittura da parte di un thread
   *
   * @param priority L'oggetto che rappresenta la priorita' da attribuire all'operazione
   * @exception SemaphoredThreadRequireException Eccezione lanciata se il thread che sta per eseguire il
   * metodo non e' un Thread di tipo <code>SemaphoredThread</code>
   */
  public void iniziaScrittura(Object priority) throws SemaphoredThreadRequireException
  { }


  /**
   * Determina la fine dell'operazione di scrittura da parte di un thread
   *
   * @exception SemaphoredThreadRequireException Eccezione lanciata se il thread che sta per eseguire il
   * metodo non e' un Thread di tipo <code>SemaphoredThread</code>
   */
  public void fineScrittura() throws SemaphoredThreadRequireException
  { }


  /**
   * Ritorna il numero di lettori attualmente in attesa della risorsa, oppure
   * in lettura sulla stessa
   *
   * @return int numero di lettori
   */
  protected int getNumReaders()
  { return numReaders;
  }


  /**
   * Incrementa il numero di lettori della risorsa
   */
  protected void incrNumReaders()
  { numReaders++;
  }


  /**
   * Decrementa il numero di lettori della risorsa
   * <p><b>ATTENZIONE</b> : Il numero di lettori non puo' essere negativo
   *
   * @return int <p> >=0 Il numero di lettori sulla risorsa
   * <p> -1 Errore, si e' cercato di decrementare il numero di lettori in maniera non consentita
   */
  protected int decrNumReaders()
  { if (numReaders == 0)
    { return -1;
    }
    else
    { numReaders--;
      return 1;
    }
  }

  /**
   * Ritorna il numero di scrittori attualmente in attesa della risorsa, oppure
   * in scrittura sulla stessa
   */
  protected int getNumWriters()
  { return numWriters;
  }


  /**
   * Incrementa il numero di scrittori della risorsa
   */
  protected void incrNumWriters()
  { numWriters++;
  }


  /**
   * Decrementa il numero di scrittori della risorsa
   * <p><b>ATTENZIONE</b> : Il numero di scrittori non puo' essere negativo
   *
   * @return int <p> >=0 Il numero di scrittori sulla risorsa
   * <p> -1 Errore, si e' cercato di decrementare il numero di scrittori in maniera non consentita
   */
  protected int decrNumWriters()
  { if (numWriters == 0)
    { return -1;
    }
    else
    { numWriters--;
      return 1;
    }
  }

}