package readerwriter;

/**
 * <p>Title:        ReaderWriter
 * <p>Description:  Progettazione di un mini framework per la gestione generale
 * di risorse condivise in maniera concorrente in lettura/scrittura
 *
 * <p>Copyright:    Copyright (c) 2001
 * <p>Company:      Universita' degli studi di Milano (Facolta' di Informatica)
 *
 * @author Gabriele "SunGray" Lana
 * @version 1.0
 */

/**
 * La classe TestFifoSelector e' una classe creata ad hoc per il test del FifoThreadSelector
 */
public class TestFifoSelector implements Runnable
{ /**
   * Test non conosciuto, ne verra' selezionato uno casualmente
   */
  public static final int UNKNOW_TEST = 0;
  /**
   * Primo tipo di test disponibile
   */
  public static final int FIRST_TEST = 1;
  /**
   * Il thread attuale viene aggiunto alla coda di Thread del FifoThreadSelector
   */
  public static final int ADD = 1;
  /**
   * Un thread scelto dall'attuale FifoThreadSelector viene rimosso dalla coda
   */
  public static final int REMOVE = 2;
  /**
   * Ultimo tipo di test disponibile
   */
  public static final int LAST_TEST = 2;

  private FifoThreadSelector selector;
  private String myName;
  private int behaviour;


  /**
   * Inizializza lo stato dell'oggetto di classe <code>TestFifoSelector</code>
   *
   * @param myName Nome che si desidera dare al flusso d'esecuzione (per il debug)
   * @param behaviour Intero che identifica il comportamento del flusso d'esecuzione, se l'intero specificato
   * non corrisponde a nessun tipo di comportamento conosciuto, ne verra' scelto uno a caso
   * @param selector Oggetto di classe <code>FifoSelector</code> che il flusso d'esecuzione dovra'
   * utilizzare al fine del test
   */
  public TestFifoSelector(String myName, int behaviour, FifoThreadSelector selector)
  { if ((behaviour < FIRST_TEST) && (behaviour > LAST_TEST))
      this.behaviour = chooseBehaviour();
    else
      this.behaviour = behaviour;
    this.myName = myName;
    this.selector = selector;
  }


  /**
   * Inizio del flusso d'esecuzione
   */
  public void run()
  { if (behaviour == ADD)
      add();
    else
      remove();
  }

  /**
   * Identifica l'oggetto attraverso il suo nome
   *
   * @return String Il nome dell'oggetto
   */
  public String toString()
  { return myName;
  }

  // Testa il selettore di Thread aggiungendosi alla lista dei Thread in attesa
  private void add()
  { while(true)
    { try
      { Thread.sleep((int)(Math.random()*5000));
      }
      catch(InterruptedException e)
      { e.printStackTrace();
      }
      StringBuffer buff = new StringBuffer();
      buff.append("Sono il thread : ").append(this).append(". Mi aggiungo alla lista");
      System.out.println(buff.toString());
      // Dato che e' un test, si presuppone che le cose siano organizzate, per questo non si fanno
      // controlli sulla correttezza del tipo di Thread
      selector.addThread((SemaphoredThread)Thread.currentThread(), null);
      buff = new StringBuffer();
      buff.append("Sono il thread : ").append(this).append(". Sono stato tolto dalla lista");
      System.out.println(buff);
    }
  }

  // Testa il selettore di Thread rimuovendo il primo Thread disponibile in attesa
  private void remove()
  { while(true)
    { try
      { Thread.sleep((int)(Math.random()*5000));
      }
      catch(InterruptedException e)
      { e.printStackTrace();
      }
      StringBuffer buff = new StringBuffer();
      buff.append("Sono il thread : ").append(this).append(". Rimuovo un thread dalla lista");
      System.out.println(buff.toString());
      selector.removeThread();
    }
  }

  // Scelta casuale del comportamento
  private int chooseBehaviour()
  { return (((int)Math.random()*100)%LAST_TEST)+1;
  }

}