package readerwriter;

/**
 * <p>Title:        ReaderWriter
 * <p>Description:  Progettazione di un mini framework per la gestione generale
 * di risorse condivise in maniera concorrente in lettura/scrittura
 *
 * <p>Copyright:    Copyright (c) 2001
 * <p>Company:      Universita' degli studi di Milano (Facolta' di Informatica)
 *
 * @author Gabriele "SunGray" Lana
 * @version 1.0
 */


/**
 * La classe <code>TestReaderWriter</code> e' una classe Runnable creata ad hoc per ottenere un flusso
 * d'esecuzione adatto per il test della classe <code>ReaderWriter</code>; la classe simula il
 * comportamento di un lettore/scrittore che vuole accedere alla risorsa controllata dal ReaderWriter
 */

public class TestReaderWriter implements Runnable
{ /**
   * Test non conosciuto, ne verra' selezionato uno casualmente
   */
  public static final int UNKNOW_TEST = 0;
  /**
   * Primo tipo di test disponibile
   */
  public static final int FIRST_TEST = 1;
  /**
   * Il thread attuale viene aggiunto alla coda di Thread del FifoThreadSelector
   */
  public static final int READER = 1;
  /**
   * Un thread scelto dall'attuale FifoThreadSelector viene rimosso dalla coda
   */
  public static final int WRITER = 2;
  /**
   * Ultimo tipo di test disponibile
   */
  public static final int LAST_TEST = 2;

  private ReaderWriter wrapper;
  private String myName;
  private int behaviour;


  /**
   * Inizializza lo stato dell'oggetto di classe <code>TestReaderWriter</code>
   *
   * @param myName Nome che si desidera dare al flusso d'esecuzione (per il debug)
   * @param behaviour Intero che identifica il comportamento del flusso d'esecuzione, se l'intero specificato
   * non corrisponde a nessun tipo di comportamento conosciuto, ne verra' scelto uno a caso
   * @param wrapper Oggetto di classe <code>ReaderWriter</code> che il flusso d'esecuzione dovra'
   * utilizzare per l'accesso alla risorsa
   */
  public TestReaderWriter(String myName, int behaviour, ReaderWriter wrapper)
  { if ((behaviour < FIRST_TEST) && (behaviour > LAST_TEST))
      this.behaviour = chooseBehaviour();
    else
      this.behaviour = behaviour;
    this.myName = myName;
    this.wrapper = wrapper;
  }


  /**
   * Inizio del flusso d'esecuzione
   */
  public void run()
  { if (behaviour == READER)
      reader();
    else
      writer();
  }

  /**
   * Identifica l'oggetto attraverso il suo nome
   *
   * @return String Il nome dell'oggetto
   */
  public String toString()
  { return myName;
  }

  // Il comportamento da Reader
  private void reader()
  { Integer priority;
    StringBuffer threadName = new StringBuffer("Sono il Thread : ");
    StringBuffer buff;
    threadName.append(myName);


    while(true)
    { buff = new StringBuffer(threadName.toString());
      priority = new Integer((int)(Math.random()*10));
      System.out.println(buff.append(" Mi accingo a leggere con una priorita' : ").append(priority));
      try
      { wrapper.leggi(priority);
      }
      catch(SemaphoredThreadRequireException e)
      { e.printStackTrace();
        System.exit(-1);
      }
      buff = new StringBuffer(threadName.toString());
      System.out.println(buff.append(" Ho finito di leggere con una priorita' : ").append(priority));
      try
      { Thread.sleep((int)(Math.random()*5000));
      }
      catch(InterruptedException e)
      { e.printStackTrace();
      }
    }
  }


  // Il comportamento da writer
  private void writer()
  { Integer priority;
    StringBuffer threadName = new StringBuffer("Sono il Thread : ");
    StringBuffer buff;
    threadName.append(myName);


    while(true)
    { buff = new StringBuffer(threadName.toString());
      priority = new Integer((int)(Math.random()*10));
      System.out.println(buff.append(" Mi accingo a scrivere con una priorita' : ").append(priority));
      try
      { wrapper.scrivi(priority);
      }
      catch(SemaphoredThreadRequireException e)
      { e.printStackTrace();
        System.exit(-1);
      }
      buff = new StringBuffer(threadName.toString());
      System.out.println(buff.append(" Ho finito di scrivere con una priorita' : ").append(priority));
      try
      { Thread.sleep((int)(Math.random()*5000));
      }
      catch(InterruptedException e)
      { e.printStackTrace();
      }
    }
  }

  // La scelta eventualmente casuale del comportamento
  private int chooseBehaviour()
  { return (((int)Math.random()*100)%LAST_TEST)+1;
  }
}






